/***************************************************************************
 * jtalk -- talk-like Jabber client
 * (C) Nicolas George - 2006
 * contacts.c: contacts database
 ***************************************************************************/

/*
  This file is part of jtalk.

  jtalk is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  jtalk is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with jtalk; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <string.h>
#include <glib.h>

#include "jtalk.h"

static GStringChunk *names;
static GHashTable *contacts_by_jid;
static GHashTable *contacts_by_short_name;

static jtalk_message_t *all_first = NULL, *all_last = NULL;

static char *
string_until(const char *s, char l, int all)
{
    char *e;
    char *r;

    e = strchr(s, l);
    if(e == NULL)
	return(all ? g_strdup(s) : NULL);
    r = g_new(char, e - s + 1);
    memcpy(r, s, e - s);
    r[e - s] = 0;
    return(r);
}

void
jtalk_contact_init(void)
{
    names = g_string_chunk_new(4096);
    contacts_by_jid = g_hash_table_new(g_str_hash, g_str_equal);
    contacts_by_short_name = g_hash_table_new(g_str_hash, g_str_equal);
}

jtalk_contact_t *
jtalk_contact_find(const char *jid, int add)
{
    char *rjid;
    jtalk_contact_t *c;

    rjid = string_until(jid, '/', 1);
    c = g_hash_table_lookup(contacts_by_jid, rjid);
    if(c == NULL && add) {
	c = g_new(jtalk_contact_t, 1);
	c->jid = g_string_chunk_insert_const(names, rjid);
	c->msg_first = c->msg_last = NULL;
	g_hash_table_insert(contacts_by_jid, c->jid, c);
    }
    g_free(rjid);
    return(c);
}

void
jtalk_contact_add_message(jtalk_contact_t *c, jtalk_message_t *m)
{
    GList *l;

    l = g_list_alloc();
    l->data = m;
    l->prev = c->msg_last;
    l->next = NULL;
    if(c->msg_last == NULL)
	c->msg_first = l;
    else
	c->msg_last->next = l;
    c->msg_last = l;

    if(m->from == NULL)
	m->to = g_slist_prepend(m->to, c);
}

jtalk_message_t *
jtalk_message_new(const char *jid, const char *text)
{
    jtalk_message_t *m;
    jtalk_contact_t *c;
    size_t l;
    char *r;

    l = strlen(text);
    m = g_malloc(sizeof(jtalk_message_t) + l + 1);
    strcpy(m->text, text);
    m->timestamp = jtalk_now();
    m->to = NULL;

    if(jid != NULL) {
	c = jtalk_contact_find(jid, 1);
	m->from = c;
	r = strchr(jid, '/');
	m->resource = r == NULL ? NULL :
	    g_string_chunk_insert_const(names, r + 1);
	jtalk_contact_add_message(c, m);
    } else {
	m->from = NULL;
	m->resource = NULL;
    }

    m->all_prev = all_last;
    m->all_next = NULL;
    if(all_last == NULL)
	all_first = m;
    else
	all_last->all_next = m;
    all_last = m;

    return(m);
}
